@@ -9,14 +9,14 @@
 Implements gameplay revision 2.0;
 Game version 2.0 ( 4+1 keys control (directions+select/menu) )
 Program version 2.0 ( ANSI C );
-Biniax-2 ver. 1.20
+Biniax-2 ver. 1.30
 
 - WHAT IS THIS :
-This program is an ANSI C version of "Biniax-2" computer game. 
+This program is an ANSI C version of "Biniax-2" computer game.
 For pure gameplay, check the Biniax Cassic game at www.biniax.com
 
 - LICENSE :
-  Biniax Copyright (C) 2005-2008 Jordan Tuzsuzov
+  Biniax Copyright (C) 2005-2009 Jordan Tuzsuzov
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
   arising from the use of this software.
@@ -107,19 +107,19 @@ int main( int argc, char *argv[] )
 		return -3;
 	if ( sndInit() == BNX_FALSE )
 		return -4;
-	
+
 	/******************************************************************
-	SHOW INITIAL WELCOME SCREEN                                                
+	SHOW INITIAL WELCOME SCREEN
 	******************************************************************/
 	gfxRenderEntry();
 
 	/******************************************************************
-	MAIN STATE MACHINE                                                
+	MAIN STATE MACHINE
 	******************************************************************/
 	while ( bquit == BNX_FALSE )
 	{
 		/******************************************************************
-		MAIN MENU STATE                                                
+		MAIN MENU STATE
 		******************************************************************/
 		if ( enterState == cStateMainMenu )
 		{
@@ -170,7 +170,7 @@ int main( int argc, char *argv[] )
 			}
 		}
 		/******************************************************************
-		HALL OF FAME VIEW STATE                                                
+		HALL OF FAME VIEW STATE
 		******************************************************************/
 		if ( enterState == cStateHallView )
 		{
@@ -178,7 +178,7 @@ int main( int argc, char *argv[] )
 			enterState = cStateMainMenu;
 		}
 		/******************************************************************
-		HALL OF FAME ENTER STATE                                                
+		HALL OF FAME ENTER STATE
 		******************************************************************/
 		if ( enterState == cStateHallEnter )
 		{
@@ -187,7 +187,7 @@ int main( int argc, char *argv[] )
 			enterState = cStateHallView;
 		}
 		/******************************************************************
-		HELP STATE                                                
+		HELP STATE
 		******************************************************************/
 		if ( enterState == cStateHelp )
 		{
@@ -195,7 +195,7 @@ int main( int argc, char *argv[] )
 			enterState = cStateMainMenu;
 		}
 		/******************************************************************
-		GAME STATE                                                
+		GAME STATE
 		******************************************************************/
 		if ( enterState == cStateGame )
 		{
@@ -241,7 +241,7 @@ void initGame( BNX_GAME *game )
 {
 	game->wins[ cPlayer1 ]		= 0;
 	game->wins[ cPlayer2 ]		= 0;
-	
+
 	initGameKeepScore( game );
 }
 
@@ -261,7 +261,7 @@ void initGameKeepScore( BNX_GAME *game )
 	game->ingame				= BNX_TRUE;
 
 	// Bad dependancy
-	game->ingame = BNX_FALSE; // NEEDED FOR initGrid() 
+	game->ingame = BNX_FALSE; // NEEDED FOR initGrid()
 	initGrid( game, cInitLines );
 	initPlayer( game ); // INIT PLAYER AFTER INIT GRID!
 	game->ingame = BNX_TRUE; // NEED TO BE AFTER initGrid()
@@ -323,10 +323,10 @@ BNX_UINT8 initPair( BNX_UINT8 maxel )
 	BNX_UINT8	left;
 	BNX_UINT8	right;
 	left = ( BNX_INT8 ) sysRand( maxel );
-	do 
+	do
 	{
 		right = ( BNX_UINT8 ) sysRand( maxel );
-	} 
+	}
 	while ( left == right );
 
 	return ((left << 4) | right);
@@ -462,7 +462,7 @@ BNX_BOOL takePair( BNX_GAME *game, BNX_INT16 x, BNX_INT16 y, BNX_INT16 pIndex )
 		return BNX_TRUE;
 	}
 
-	if ( pairLeft( pair ) == game->player[ pIndex ].e ) 
+	if ( pairLeft( pair ) == game->player[ pIndex ].e )
 	{
 		game->player[ pIndex ].e = pairRight( pair );
 		cantake = BNX_TRUE;
@@ -569,7 +569,7 @@ void correctSpeed( BNX_GAME *game )
 		BNX_INT32 scrollRange = cMaxScroll - cMinScroll;
 		BNX_INT32 speedIndex = (scrollRange << 8) / cMaxSpeedScore;
 		BNX_INT16 pIndex = game->score[ cPlayer1 ] > game->score[ cPlayer2 ] ? cPlayer1 : cPlayer2;
-		
+
 		if ( game->score[ pIndex ] < cMaxSpeedScore )
 		{
 			game->speed = cMaxScroll - ( ( game->score[ pIndex ] * speedIndex ) >> 8 );
@@ -623,14 +623,14 @@ GAME SESSION
 BNX_INT16 gameSession( BNX_GAME *game )
 {
 	static BNX_INT16	countTakes = 0;
-	BNX_INT16			nTemp;
+	BNX_INT16			nTemp = 0;
 	BNX_BOOL			bgameSes = BNX_TRUE;
 	BNX_INT16			naction = cDoSave;
-	BNX_INT32			startTime;
-	BNX_INT32			prevScore;
-	BNX_BOOL			flagTake;
-	BNX_BOOL			flagMove;
-	BNX_BOOL			prevIngame;
+	BNX_INT32			startTime = 0;
+	BNX_INT32			prevScore = 0;
+	BNX_BOOL			flagTake = BNX_FALSE;
+	BNX_BOOL			flagMove = BNX_FALSE;
+	BNX_BOOL			prevIngame = BNX_FALSE;
 
 	inpInit();
 	while ( bgameSes )
@@ -642,6 +642,7 @@ BNX_INT16 gameSession( BNX_GAME *game )
 		game->sounds = cSndNone;
 
 		inpUpdate();
+		gfxGetVirtualKey( game, inpDirect() );
 		if ( inpKeyA() )
 		{
 			switch ( game->mode )
@@ -667,7 +668,7 @@ BNX_INT16 gameSession( BNX_GAME *game )
 		{
 			bgameSes = BNX_FALSE;
 			naction = cDoNothing;
-		
+
 			if ( game->mode != cModeMultiplayer )
 			{
 				if ( game->ingame == BNX_TRUE )
@@ -699,7 +700,6 @@ BNX_INT16 gameSession( BNX_GAME *game )
 
 		if ( game->ingame == BNX_TRUE )
 		{
-
 			switch ( game->mode )
 			{
 				/***************************************************************
@@ -1010,7 +1010,7 @@ BNX_INT16 gameSession( BNX_GAME *game )
 					{
 						game->sounds |= soundMask( cSndFail );
 					}
-				
+
 					// Detect shake start. Don't *play* shake at high speed
 					if ( game->scroll == cShakeAfter && game->score[ cPlayer1 ] < cMaxSpeedScore )
 					{
@@ -1084,14 +1084,29 @@ MAIN MENU
 
 BNX_INT16 mainMenu( BNX_GAME *game )
 {
-	BNX_INT16 option = 0;
-	BNX_INT32 startTime;
+	BNX_INT16	option = 0;
+	BNX_INT16	gfxoption = -1;
+	BNX_INT32	startTime;
+	BNX_BOOL	penchoice = BNX_FALSE;
 
 	inpInit();
 	do
 	{
 		startTime = sysGetTime();
 		inpUpdate();
+		gfxoption = gfxGetMenuOption( inpDirect() );
+		if ( gfxoption >= 0 )
+		{
+			if ( option == gfxoption )
+			{
+				penchoice = BNX_TRUE;
+			}
+			else
+			{
+				option = gfxoption;
+			}
+		}
+
 		if ( inpKeyLeft() || inpKeyUp() )
 		{
 			option = ( option + cMaxOptions - 1 ) % cMaxOptions;
@@ -1114,7 +1129,7 @@ BNX_INT16 mainMenu( BNX_GAME *game )
 			sysUpdate();
 		}
 	}
-	while ( inpKeyA() == BNX_FALSE );
+	while ( (inpKeyA() == BNX_FALSE) && (penchoice == BNX_FALSE) );
 
 	return option;
 }
@@ -1122,11 +1137,13 @@ BNX_INT16 mainMenu( BNX_GAME *game )
 void showHelp( BNX_GAME *game )
 {
 	BNX_INT16 line = 0;
+	BNX_INT16 virtKey = 0;
 
 	inpInit();
 	do
 	{
 		inpUpdate();
+		gfxGetHelpPen( inpDirect() );
 		if ( inpKeyLeft() || inpKeyUp() )
 		{
 			line --;
@@ -1164,13 +1181,55 @@ GAME AND HISCORE SAVE / RESTORE
 BNX_BOOL saveGame( BNX_GAME *game )
 {
 	FILE		*file;
+	BNX_INT32	i;
+	BNX_INT32	j;
 
-	file = fopen( "autosave.bnx2", "wb" );
+	file = fopen( sysGetFullFileName( csSaveGameName ), "wb" );
 
 	if ( file == (FILE *) NULL )
 		return BNX_FALSE;
 
-	fwrite( game, 1, sizeof( BNX_GAME ), file );
+	sysFPut32( game->moment, file );
+	for ( i = 0; i < cMaxPlayers; ++i )
+	{
+		sysFPut8( game->player[ i ].x, file );
+		sysFPut8( game->player[ i ].y, file );
+		sysFPut8( game->player[ i ].e, file );
+	}
+	for ( j = 0; j < cGridX; ++j )
+	{
+		for ( i = 0; i < cGridY; ++i )
+		{
+			sysFPut8( game->grid[ j ][ i ], file );
+		}
+	}
+	sysFPut1byte( file );
+	sysFPut16( game->mode, file );
+	for ( i = 0; i < cMaxPlayers; ++i )
+	{
+		sysFPut32( game->score[ i ], file );
+	}
+	for ( i = 0; i < cMaxPlayers; ++i )
+	{
+		sysFPut32( game->wins[ i ], file );
+	}
+	for ( i = 0; i < cMaxModes; ++i )
+	{
+		sysFPut32( game->best[ i ], file );
+	}
+	sysFPut16( game->scroll, file );
+	sysFPut16( game->speed, file );
+	sysFPut16( game->moves, file );
+	sysFPut16( game->clears, file );
+	sysFPut8( game->ingame, file );
+	sysFPut3byte( file );
+	sysFPut32( game->sounds, file );
+	sysFPut8( game->message, file );
+	sysFPut3byte( file );
+	sysFPut3byte( file );
+	sysFPut1byte( file );
+	sysFPut16( game->level, file );
+	sysFPut16( game->level_count, file );
 
 	fclose( file );
 
@@ -1180,13 +1239,56 @@ BNX_BOOL saveGame( BNX_GAME *game )
 BNX_BOOL loadGame( BNX_GAME *game )
 {
 	FILE		*file;
+	BNX_INT32	i;
+	BNX_INT32	j;
 
-	file = fopen( "autosave.bnx2", "rb" );
-
+	if ( sysGetFileLen( sysGetFullFileName( csSaveGameName ) ) != cSaveFileSize )
+		return BNX_FALSE;
+	file = fopen( sysGetFullFileName( csSaveGameName ), "rb" );
 	if ( file == (FILE *) NULL )
 		return BNX_FALSE;
 
-	fread( game, 1, sizeof( BNX_GAME ), file );
+	game->moment = sysFGet32( file );
+	for ( i = 0; i < cMaxPlayers; ++i )
+	{
+		game->player[ i ].x = sysFGet8( file );
+		game->player[ i ].y = sysFGet8( file );
+		game->player[ i ].e = sysFGet8( file );
+	}
+	for ( j = 0; j < cGridX; ++j )
+	{
+		for ( i = 0; i < cGridY; ++i )
+		{
+			game->grid[ j ][ i ] = sysFGet8( file );
+		}
+	}
+	sysFGet1byte( file );
+	game->mode = sysFGet16( file );
+	for ( i = 0; i < cMaxPlayers; ++i )
+	{
+		game->score[ i ] = sysFGet32( file );
+	}
+	for ( i = 0; i < cMaxPlayers; ++i )
+	{
+		game->wins[ i ] = sysFGet32( file );
+	}
+	for ( i = 0; i < cMaxModes; ++i )
+	{
+		game->best[ i ] = sysFGet32( file );
+	}
+	game->scroll = sysFGet16( file );
+	game->speed = sysFGet16( file );
+	game->moves = sysFGet16( file );
+	game->clears = sysFGet16( file );
+	game->ingame = sysFGet8( file );
+	sysFGet3byte( file );
+	game->sounds = sysFGet32( file );
+	game->message = sysFGet8( file );
+	sysFGet3byte( file );
+	sysFGet3byte( file );
+	sysFGet1byte( file );
+	game->level = sysFGet16( file );
+	game->level_count = sysFGet16( file );
 
 	fclose( file );
 
