@@ -1,6 +1,6 @@
 /******************************************************************************
 BINIAX GRAPHICS-RELATED IMPLEMENTATIONS
-COPYRIGHT JORDAN TUZSUZOV, (C) 2005-2007
+COPYRIGHT JORDAN TUZSUZOV, (C) 2005-2009
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -494,13 +494,14 @@ void gfxRenderHelp( BNX_INT16 *line )
 	*line = l;
 }
 
-void gfxRenderHof( BNX_HALL *hof )
+void gfxRenderHof( BNX_HALL *hof, BNX_INT16 hofview )
 {
 	static BNX_INT16 count = 0;
 
 	BNX_INT16	i;
 	BNX_INT16	x;
 	BNX_INT16	y;
+	SDL_Rect	pos;
 
 	char ptrTxt[ cGfxStrLen ];
 
@@ -534,6 +535,25 @@ void gfxRenderHof( BNX_HALL *hof )
 		y += cGfxHelpDY;
 	}
 
+	if ( cfgGetTouch() == BNX_TRUE )
+	{
+		switch ( hofview )
+		{
+			case cGfxHofKeyboardUp:
+				pos.x = cGfxKeyboardX;
+				pos.y = cGfxKeyboardY2;
+				SDL_BlitSurface( _Gfx.keyboard, NULL, _Gfx.screen, &pos );
+				break;
+			case cGfxHofKeyboardDn:
+				pos.x = cGfxKeyboardX;
+				pos.y = cGfxKeyboardY1;
+				SDL_BlitSurface( _Gfx.keyboard, NULL, _Gfx.screen, &pos );
+				break;
+			default:
+				break;
+		}
+	}
+
 	if ( (count % cGfxFall) == 0 )
 	{
 		gfxNewParticle( sysRand( cGfxScreenX ), sysRand( cGfxScreenY ) );
@@ -570,9 +590,9 @@ void gfxPrintText( BNX_INT16 x, BNX_INT16 y, const char *text )
 
 void gfxPrintTextWave( BNX_INT16 x, BNX_INT16 y, const char *text, BNX_INT16 start )
 {
-	char		c;
-	SDL_Rect	tpos, ppos;
-	BNX_INT16	sinuz[] = { 0, 1, 2, 3, 4, 3, 2, 1, 0, -1, -2, -3, -4, -3, -2, -1 };
+	char				c;
+	SDL_Rect			tpos, ppos;
+	static BNX_INT16	sinuz[] = { 0, 1, 2, 3, 4, 3, 2, 1, 0, -1, -2, -3, -4, -3, -2, -1 };
 
 	ppos.x = x;
 	ppos.y = y;
@@ -789,33 +809,34 @@ BNX_BOOL gfxLoadResources()
 	BNX_BOOL bLoad = BNX_TRUE;
 
 	/* Load GUI related graphics data */
-	bLoad |= gfxLoadImage( "data/graphics/background0.png", &_Gfx.background[ cModeRealtime ]);
-	bLoad |= gfxLoadImage( "data/graphics/background1.png", &_Gfx.background[ cModeTurn ]);
-	bLoad |= gfxLoadImage( "data/graphics/background2.png", &_Gfx.background[ cModeMultiplayer ] );
+	bLoad &= gfxLoadImage( "data/graphics/background0.png", &_Gfx.background[ cModeRealtime ]);
+	bLoad &= gfxLoadImage( "data/graphics/background1.png", &_Gfx.background[ cModeTurn ]);
+	bLoad &= gfxLoadImage( "data/graphics/background2.png", &_Gfx.background[ cModeMultiplayer ] );
 
-	bLoad |= gfxLoadImage( "data/graphics/window.png", &_Gfx.window );
-	bLoad |= gfxLoadImage( "data/graphics/splash.png", &_Gfx.splash );
-	bLoad |= gfxLoadImage( "data/graphics/logo.png", &_Gfx.logo );
-	bLoad |= gfxLoadImage( "data/graphics/help.png", &_Gfx.help );
-	bLoad |= gfxLoadImage( "data/graphics/font.png", &_Gfx.font );
-	bLoad |= gfxLoadImage( "data/graphics/roadmap.png", &_Gfx.roadmap );
+	bLoad &= gfxLoadImage( "data/graphics/window.png", &_Gfx.window );
+	bLoad &= gfxLoadImage( "data/graphics/splash.png", &_Gfx.splash );
+	bLoad &= gfxLoadImage( "data/graphics/logo.png", &_Gfx.logo );
+	bLoad &= gfxLoadImage( "data/graphics/help.png", &_Gfx.help );
+	bLoad &= gfxLoadImage( "data/graphics/font.png", &_Gfx.font );
+	bLoad &= gfxLoadImage( "data/graphics/roadmap.png", &_Gfx.roadmap );
 
 	// Loading Game Elements
-	bLoad |= gfxLoadImage( "data/graphics/element0.png", &_Gfx.elements[ 0 ] );
-	bLoad |= gfxLoadImage( "data/graphics/element1.png", &_Gfx.elements[ 1 ] );
-	bLoad |= gfxLoadImage( "data/graphics/element2.png", &_Gfx.elements[ 2 ] );
-	bLoad |= gfxLoadImage( "data/graphics/element3.png", &_Gfx.elements[ 3 ] );
-	bLoad |= gfxLoadImage( "data/graphics/elementB.png", &_Gfx.elements[ 4 ] );
+	bLoad &= gfxLoadImage( "data/graphics/element0.png", &_Gfx.elements[ 0 ] );
+	bLoad &= gfxLoadImage( "data/graphics/element1.png", &_Gfx.elements[ 1 ] );
+	bLoad &= gfxLoadImage( "data/graphics/element2.png", &_Gfx.elements[ 2 ] );
+	bLoad &= gfxLoadImage( "data/graphics/element3.png", &_Gfx.elements[ 3 ] );
+	bLoad &= gfxLoadImage( "data/graphics/elementB.png", &_Gfx.elements[ 4 ] );
 
-	bLoad |= gfxLoadImage( "data/graphics/movecount.png", &_Gfx.movecount );
-	bLoad |= gfxLoadImage( "data/graphics/cursor0.png", &_Gfx.cursors[ 0 ] );
-	bLoad |= gfxLoadImage( "data/graphics/cursor1.png", &_Gfx.cursors[ 1 ] );
-	bLoad |= gfxLoadImage( "data/graphics/marker0.png", &_Gfx.markers[ 0 ] );
-	bLoad |= gfxLoadImage( "data/graphics/marker1.png", &_Gfx.markers[ 1 ] );
+	bLoad &= gfxLoadImage( "data/graphics/movecount.png", &_Gfx.movecount );
+	bLoad &= gfxLoadImage( "data/graphics/keyboard.png", &_Gfx.keyboard );
+	bLoad &= gfxLoadImage( "data/graphics/cursor0.png", &_Gfx.cursors[ 0 ] );
+	bLoad &= gfxLoadImage( "data/graphics/cursor1.png", &_Gfx.cursors[ 1 ] );
+	bLoad &= gfxLoadImage( "data/graphics/marker0.png", &_Gfx.markers[ 0 ] );
+	bLoad &= gfxLoadImage( "data/graphics/marker1.png", &_Gfx.markers[ 1 ] );
 
-	bLoad |= gfxLoadImage( "data/graphics/particle0.png", &_Gfx.part[ 0 ] );
-	bLoad |= gfxLoadImage( "data/graphics/particle1.png", &_Gfx.part[ 1 ] );
-	bLoad |= gfxLoadImage( "data/graphics/particle2.png", &_Gfx.part[ 2 ] );
+	bLoad &= gfxLoadImage( "data/graphics/particle0.png", &_Gfx.part[ 0 ] );
+	bLoad &= gfxLoadImage( "data/graphics/particle1.png", &_Gfx.part[ 1 ] );
+	bLoad &= gfxLoadImage( "data/graphics/particle2.png", &_Gfx.part[ 2 ] );
 
 	// Set alpha
 	SDL_SetAlpha( _Gfx.window, SDL_SRCALPHA | SDL_RLEACCEL, cGfxWinBlending );
@@ -826,6 +847,167 @@ BNX_BOOL gfxLoadResources()
 	return bLoad;
 }
 
+/******************************************************************************
+INPUT VIA TOUCH-SCREEN. NOT THE NICEST INCLUDE IN THE ARCHITECURE...
+******************************************************************************/
+
+BNX_BOOL gfxInBox( BNX_INT16 x, BNX_INT16 y, BNX_BOX *box )
+{
+	if ( x >= box->x1 && y >= box->y1 && x <= box->x2 && y <= box->y2 )
+	{
+		return BNX_TRUE;
+	}
+	else
+	{
+		return BNX_FALSE;
+	}
+}
+
+void gfxGetVirtualKey( BNX_GAME *game, BNX_INP *inp )
+{
+	BNX_INT16	nPlayerCenterX;
+	BNX_INT16	nPlayerCenterY;
+	
+	if ( inp->mousePress == BNX_TRUE && cfgGetTouch() == BNX_TRUE )
+	{
+		inp->mousePress = BNX_FALSE;
+
+		/* Proceed with ingame handling */
+		if ( gfxInBox( inp->mouseX, inp->mouseY, &(_BNX_BOXES[ cGfxIndField ]) ) == BNX_TRUE )
+		{
+			nPlayerCenterX = 
+				cGfxZeroX + game->player[ cPlayer1 ].x * cGfxPairPlusX + (cGfxPairPlusX>>1);
+			nPlayerCenterY = 
+				cGfxZeroY + game->player[ cPlayer1 ].y * cGfxPairPlusY - (cGfxPairPlusY>>1);
+
+			if ( abs( nPlayerCenterX - inp->mouseX ) > abs( nPlayerCenterY - inp->mouseY ) )
+			{
+				/* Move horizontally */
+				if ( nPlayerCenterX > inp->mouseX )
+				{
+					inp->keyLeft = BNX_TRUE;
+				}
+				else
+				{
+					inp->keyRight = BNX_TRUE;
+				}
+			}
+			else
+			{
+				/* Move vertically */
+				if ( nPlayerCenterY > inp->mouseY )
+				{
+					inp->keyUp = BNX_TRUE;
+				}
+				else
+				{
+					inp->keyDown = BNX_TRUE;
+				}
+			}
+		}
+		/* Detect Ingame Escape */
+		else  if ( gfxInBox( inp->mouseX, inp->mouseY, &(_BNX_BOXES[ cGfxIndEscape ]) ) == BNX_TRUE )
+		{
+			inp->keyB = BNX_TRUE;
+		}
+		/* Try to detect Ingame Space press */
+		else
+		{
+			if ( game->mode == cModeRealtime )
+			{
+				if ( gfxInBox( inp->mouseX, inp->mouseY, &(_BNX_BOXES[ cGfxIndSpaceR ]) ) == BNX_TRUE )
+				{
+					inp->keyC = BNX_TRUE;
+				}
+			}
+			else if ( game->mode == cModeTurn )
+			{
+				if ( gfxInBox( inp->mouseX, inp->mouseY, &(_BNX_BOXES[ cGfxIndSpaceT ]) ) == BNX_TRUE )
+				{
+					inp->keyC = BNX_TRUE;
+				}
+			}
+		}
+	}
+}
+
+void gfxGetVirtualChar( BNX_GAME *game, BNX_INP *inp )
+{
+	BNX_INT32	arx, ary;
+	char		c = 0;
+
+	if ( inp->mousePress == BNX_TRUE && cfgGetTouch() == BNX_TRUE )
+	{
+		inp->mousePress = BNX_FALSE;
+		arx = ((inp->mouseX - cGfxKeyboardX) * cGfxKeyGridX) / _Gfx.keyboard->w;
+		switch ( game->mode )
+		{
+			case cModeTurn:
+				ary = (((inp->mouseY - cGfxKeyboardY2) * cGfxKeyGridY) / _Gfx.keyboard->h);
+				break;
+			case cModeRealtime:
+				ary = (((inp->mouseY - cGfxKeyboardY1) * cGfxKeyGridY) / _Gfx.keyboard->h);
+				break;
+			default:
+				return;
+		}
+		if ( arx >=0 && arx < cGfxKeyGridX && ary >= 0 && ary < cGfxKeyGridY )
+		{
+			c = virtualKBD[ ary ][ arx ];
+			switch (c)
+			{
+				case 'e':
+					inp->keyA = BNX_TRUE;
+					break;
+				case 'b':
+					inp->keyDel = BNX_TRUE;
+					break;
+				default:
+					inp->letter = c;
+					break;
+			}
+		}
+	}
+}
+
+BNX_INT16 gfxGetMenuOption( BNX_INP *inp )
+{
+	BNX_INT16 i;
+
+	if ( inp->mousePress == BNX_TRUE && cfgGetTouch() == BNX_TRUE )
+	{
+		inp->mousePress = BNX_FALSE;
+		for ( i = 0; i < cMaxOptions; ++i )
+		{
+			if ( gfxInBox( inp->mouseX, inp->mouseY, &(_BNX_MENU_BOXES[ i ]) ) == BNX_TRUE )
+			{
+				return i;
+			}
+		}
+	}
+	return -1;
+}
+
+void gfxGetHelpPen( BNX_INP *inp )
+{
+	if ( inp->mousePress == BNX_TRUE && cfgGetTouch() == BNX_TRUE )
+	{
+		inp->mousePress = BNX_FALSE;
+		if ( gfxInBox( inp->mouseX, inp->mouseY, &(_BNX_BOXES[ cGfxIndEscape ]) ) == BNX_TRUE )
+		{
+			inp->keyA = BNX_TRUE;
+		}
+		else if ( inp->mouseY < (cGfxScreenY>>1) )
+		{
+			inp->keyPageUp = BNX_TRUE;
+		}
+		else if ( inp->mouseY > (cGfxScreenY>>1) )
+		{
+			inp->keyPageDown = BNX_TRUE;
+		}
+	}
+}
+
 /******************************************************************************
 PARTICLE SYSTEM
 ******************************************************************************/
